# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.11.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ### Overview
#
# This notebook implements a processing pipeline from computes word co-occurrences from plain text. The term-term co-occurrence matrix is transformed into a DTM corpus that has a vocabulary consisting of token-pairs. The co-occurring word trends can hence be xxplored using the ordinary word trends analysis tools.
#
# For large corpora the processing time can be considerable and in such a case you
# should consider using the CLI-version of the processing pipeline.
#
# ### Text Processing Pipeline
#
# | | Building block | Arguments | Description | Configuration
# | -- | :------------- | :------------- | :------------- | :------------- |
# | ⚙ | <b>SetTagger</b>~~SpacyModel~~ | tagger | Set PoS tagger | spaCy
# | 📜| <b>LoadText</b> | reader_opts, transform_opts | Load text stream | config.yaml
# | 🔎 | <b>Tqdm</b> | ⚪ | Progress indicator | ⚪
# | ⌛ | <b>Passthrough</b> | ⚪ | Passthrough  | ⚪
# | 🔨 | Spacy<b>ToTaggedFrame</b> | tagger service | PoS tagging |
# | 💾 | <b>Checkpoint</b> | checkpoint_filename | Checkpoint (tagged frames) to file |
# | 🔨 | TaggedFrame<b>ToTokens</b> | extract_tagged_tokens_opts, filter_opts | Tokens extractor | User
# | 🔨 | <b>TokensTransform</b> | tokens_transform_opts | Tokens transformer | User
# | 🔨 | <b>Vocabulary</b> | ⚪ | Generate a token to integer ID mapping | ⚪
# | 🔨 | <b>ToDocumentContentTuple</b> | ⚪ | API adapter| ⚪
# | 🔨 | <i>Partition</i> | ⚪ | Partition corpus into subsets based on predicate | 'year'
# | 🔎 | <i>ToTTM</i> | ⚪ | Transform each partition to TTM matrices | User
# | 🔨 | <b>ToCoOccurrence</b> | ⚪| Transform TTM into data frame with normalized values | ⚪
# | 💾 | <i>Checkpoint</i> | checkpoint_filename| Store co-occurrence data frame | ⚪
# | 🔨 | <b>ToDTM</b> | vectorize_opts| Transform data frame into DTM | ⚪
# | 💾 | <b>Checkpoint</b> | checkpoint_filename| Checkpoint (DTM) to file | ⚪
#
#
# ### How the co-occurrence counts are computed
#
# The co-occurrences are computed using a sliding window of size (D + 1 + D) that word by word moves through each document in the corpus, and keeps count of how many windows each pair of words co-occur in. Note that all windows will (currenty) always have an odd number of words, and the reason for this is the conditioned co-occurrence described below.
#
# The computation is done by first creating a (streamed) windowed corpus, consisting of all windows. From this corpus a DTM (document-term-matrix) is created, giving counts for each word in each window. This DTM is then used to compute a TTM (term-term-matrix) simply by multiplying the DTM with a transposed version of itself.
#
# Also note that the process of generating windows currently ignores sentences, paragraphs etc.
#
# #### GUI elements
#
# | | Config element |  Description |
# | -- | :------------- | :------------- |
# | | Corpus type | Type of corpus, disabled since only text corpora are allowed in this notebook.
# | | Source corpus file | Select file (ZIP) or folder that contains the text documents.
# | | Output tag | String that will be prefix to result files. Only valid filename chars are allowed.
# | | Output folder | Target folder for result files.
# | | Part-of-speech groups | Groups of tags to include in DTM given corpus PoS-schema
# | | Filename fields | Specifies attribute values to be extracted from filenames
# | | Lemmatize | Use word's lemma (base form)
# | | To lower | Lowercase all words
# | | Only alphabetic | Filter out words that has at least one non-alphabetic character
# | | Only alphanumeric | Filter out words that has at least one non-alphanumeric character
# | | No stopwords | Remove common stopwords using NLTK language specific stopwords
# | | Extra stopwords | Additional stopwords to remove
# | | Create subfolder | Sore result in subfolder named `output tag` in `target folder`
# | | Context distance | Max allowed distance to window's center-most word
# | | Concept | If specified, filter out windows having a center-most word not in list of specified words
# | | Remove concept | If specified, filter out concept word's co-occurrence pairs
# | | Normalize | Normalize output based on selected time period's corpus size
# | | Smooth | Smooth curve using PCHIP interpolations.
#
#
# N.B. Note that PoS schema (e.g. SUC, Universal, ON5 Penn Treebank tag sets) and language must be set for each corpus. This, and other options, is specified in the _corpus configuration file_. For an example, please see _SSI.yaml_ inf the `resources` folder.
#
#
# ### Concept co-occurrence
#
# The algorithm allows for computing a conditioned co-occurrence, where the set of windows are constrained so that the center-most word must one of a number of specified (concept) words. This results in a set of co-occurrences that occur in close proximity (i.e. the max distance of D) of the center-most word.
#
# ### Word trends
#
# Specifiy words of interest in the textbox in the text box. You can use both __wildcards__ ```*``` and __regular expressions__ to widen your search. The
# words in the vocabulary that matches what you have specified will be listed in the selection box. Since using wildcards and regexps can result
# in a large number of words, only the `Word count` most frequent words are displayed. Refine your search if you get to many matches.
#
# The words are plotted when selected in the selection box. You can select and plot __multiple words__ by pressing CTRL, or using CTRL + arrow keys. The words are listed  in descending order by global corpus frequency.
#
# The regular expressions must be surrounded by vertical bars ```|```. To find words ending with ```tion```
# you can enter ```|.*tion$|```  The vertical
# bars is specified only so that the system can distinguish the regexp from normal words. The actual expression is ```^.*tion$```.
# The dot and star ```.*``` matches any character (the dot) any number of times (the ```*```). The dollar sign ```$``` indicates the word ending.
# So this expression matches all words that begins with any number of characters follwoed, by the character sequence ```tion``` at the end of the word.
# To match all words starting with `info`you can enter ```|^info.*|``` where ```^``` specifies the start of the word.
#
# The "words" in this case are co-occurrence pairs and to find instances matching "information" you could enter ```information*```, ```*information``` or ```*information*``` to match pairs starting with information, ending with information or containing information respectively.

# %%

from bokeh.plotting import output_notebook
from IPython.core.display import display
from penelope.notebook.co_occurrence import main_gui

import __paths__

output_notebook()
gui = main_gui.MainGUI(
    corpus_config="riksdagens-protokoll",
    corpus_folder=__paths__.corpus_folder,
    data_folder=__paths__.data_folder,
    resources_folder=__paths__.resources_folder,
    global_count_threshold=1,
)
display(gui.layout())

# %%



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Word Trend Analysis
#
# ### Load previously vectorized corpus
#
# Use the `vectorize_protocol` script to create a new corpus with different settings.

# %%
# %load_ext autoreload
# %autoreload 2

# %%

from bokeh.plotting import output_notebook
from IPython.core.display import display
from penelope import pipeline, workflows
from penelope.corpus import dtm
from penelope.notebook import interface, word_trends
from penelope.notebook.dtm import load_dtm_gui, to_dtm_gui

import __paths__

output_notebook()

# %% tags=[] vscode={"end_execution_time": "2020-08-31T18:34:55.995Z", "start_execution_time": "2020-08-31T18:34:55.854Z"}


def done_callback(corpus: dtm.VectorizedCorpus, corpus_folder: str, corpus_tag: str):

    trends_data: word_trends.TrendsData = word_trends.TrendsData(
        corpus=corpus,
        corpus_folder=corpus_folder,
        corpus_tag=corpus_tag,
        n_count=25000,
    ).update()

    gui = word_trends.GofTrendsGUI(
        gofs_gui=word_trends.GoFsGUI().setup(), trends_gui=word_trends.TrendsGUI().setup(displayers=[])
    )

    display(gui.layout())
    gui.display(trends_data=trends_data)


def compute_done_callback(corpus: dtm.VectorizedCorpus, opts: interface.ComputeOpts):
    done_callback(corpus=corpus, corpus_folder=opts.target_folder, corpus_tag=opts.corpus_tag)


def compute_callback(args: interface.ComputeOpts, corpus_config: pipeline.CorpusConfig) -> dtm.VectorizedCorpus:
    corpus: dtm.VectorizedCorpus = workflows.document_term_matrix.compute(args=args, corpus_config=corpus_config)
    return corpus


compute_gui: to_dtm_gui.ComputeGUI = to_dtm_gui.create_compute_gui(
    corpus_config="riksdagens-protokoll",
    corpus_folder=__paths__.corpus_folder,
    data_folder=__paths__.data_folder,
    compute_callback=compute_callback,
    done_callback=compute_done_callback,
)
display(compute_gui.layout())

# %%
load_gui: load_dtm_gui.LoadGUI = load_dtm_gui.create_load_gui(
    corpus_folder=__paths__.data_folder,
    loaded_callback=done_callback,
)
display(load_gui.layout())



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# # Token Count Statistics
# ### Text Processing Pipeline
#
# | | Building block | Arguments | Description |
# | -- | :------------- | :------------- | :------------- |
# | 💾 | <b>Checkpoint</b> | checkpoint_filename | Checkpoint (tagged frames) to file
#
# The PoS tagging notebook uses the same processing pipeline as the Word trends tnotebook  do to produce a tagged data frame. The processing will henceread
# a checkpoint file if it exists, otherwise resolve the full pipeline.
#
# The word count statistics are collected in the tagging task (part-of-speech and lemma annotation). The computed statistics, total word count and the word counts for each PoS-grouping, are added (or updated) to the _document index file_ as new columns. This file is stored in the tagged text archive as `document_index.csv`.
#
# Note: The dcument index file is either a pre-existing document index or, if no such index exists, automatically generated during the initial text loading pipeline task.
# If no pre-existing file exists, then the necessary attributes (e.g. document's year) are extracted from the filename of each  document.

# %% tags=[]
from IPython.core.display import display
from penelope.notebook.token_counts import tokens_count_gui

import __paths__  # pylint: disable=unused-import

gui = tokens_count_gui.create_token_count_gui(
    corpus_folder=__paths__.corpus_folder,
    resources_folder=__paths__.resources_folder,
)
display(gui.layout())
# %%



import warnings
from dataclasses import dataclass

import ipywidgets as widgets
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display
from penelope.notebook.topic_modelling import TopicModelContainer, display_topic_trends

import notebooks.political_in_newspapers.corpus_data as corpus_data

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)

TEXT_ID = 'topic_share_plot'


@dataclass
class GUI:  # pylint: disable=too-many-instance-attributes

    text = widgets_utils.text_widget(TEXT_ID)
    n_topics = None
    text_id = TEXT_ID

    publication_id = widgets.Dropdown(
        description='Publication',
        options=utility.extend(dict(corpus_data.PUBLICATION2ID), {'(ALLA)': None}),
        value=None,
        layout=widgets.Layout(width="200px"),
    )
    aggregate = widgets.Dropdown(
        description='Aggregate',
        options=[(x['description'], x['key']) for x in topic_modelling.YEARLY_MEAN_COMPUTE_METHODS],
        value='true_mean',
        layout=widgets.Layout(width="200px"),
    )
    normalize = widgets.ToggleButton(description='Normalize', value=True, layout=widgets.Layout(width="120px"))
    topic_id = widgets.IntSlider(description='Topic ID', min=0, max=999, step=1, value=0, continuous_update=False)
    output_format = widgets.Dropdown(
        description='Format', options=['Chart', 'Table'], value='Chart', layout=widgets.Layout(width="200px")
    )
    progress = widgets.IntProgress(min=0, max=4, step=1, value=0)
    output = widgets.Output()
    prev_topic_id = None
    next_topic_id = None

    def layout(self):
        return widgets.VBox(
            [
                widgets.HBox(
                    [
                        widgets.VBox(
                            [
                                widgets.HBox([self.prev_topic_id, self.next_topic_id]),
                                self.progress,
                            ]
                        ),
                        widgets.VBox([self.topic_id]),
                        widgets.VBox([self.publication_id]),
                        widgets.VBox([self.aggregate, self.output_format]),
                        widgets.VBox([self.normalize]),
                    ]
                ),
                self.text,
                self.output,
            ]
        )


def display_gui(state: TopicModelContainer, extra_filter=None):  # pylint: disable=unused-argument

    gui = GUI()
    gui.topic_id.max = state.num_topics - 1
    gui.prev_topic_id = widgets_utils.button_with_previous_callback(gui, 'topic_id', state.num_topics)
    gui.next_topic_id = widgets_utils.button_with_next_callback(gui, 'topic_id', state.num_topics)

    def on_topic_change_update_gui(topic_id):

        if gui.n_topics != state.num_topics:
            gui.n_topics = state.num_topics
            gui.topic_id.value = 0
            gui.topic_id.max = state.num_topics - 1

        tokens = topic_modelling.get_topic_title(state.inferred_topics.topic_token_weights, topic_id, n_tokens=200)

        gui.text.value = 'ID {}: {}'.format(topic_id, tokens)

    _current_weight_over_time = dict(publication_id=-1, weights=None)

    def weight_over_time(document_topic_weights, publication_id):
        """Cache weight over time due to the large number of ocuments"""
        if _current_weight_over_time["publication_id"] != publication_id:
            _current_weight_over_time["publication_id"] = publication_id
            df = document_topic_weights
            if publication_id is not None:
                df = df[df.publication_id == publication_id]
            _current_weight_over_time["weights"] = topic_modelling.compute_topic_yearly_means(df).fillna(0)

        return _current_weight_over_time["weights"]

    def update_handler(*_):

        gui.output.clear_output()

        with gui.output:

            on_topic_change_update_gui(gui.topic_id.value)

            weights = weight_over_time(state.inferred_topics.document_topic_weights, gui.publication_id.value)

            display_topic_trends(
                weight_over_time=weights,
                topic_id=gui.topic_id.value,
                year_range=state.inferred_topics.year_period,
                aggregate=gui.aggregate.value,
                normalize=gui.normalize.value,
                output_format=gui.output_format.value,
            )

    gui.topic_id.observe(update_handler, names='value')
    gui.publication_id.observe(update_handler, names='value')
    gui.normalize.observe(update_handler, names='value')
    gui.aggregate.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')

    display(gui.layout)

    update_handler()



import os
import sys


def find_root_folder(x: str) -> str:
    return os.path.join(os.getcwd().split(x)[0], x)


project_name: str = 'welfare_state_analytics'

if os.environ.get("JUPYTER_IMAGE_SPEC", "") != "":
    root_folder: str = f"/home/jovyan/work/{project_name}"
    corpus_folder: str = "/data/westac"
else:
    root_folder: str = find_root_folder(project_name)
    corpus_folder: str = os.path.join(root_folder, "data")

if root_folder not in sys.path:
    sys.path.insert(0, root_folder)

ROOT_FOLDER: str = root_folder

data_folder: str = os.path.join(root_folder, "data")
resources_folder: str = os.path.join(root_folder, "resources")



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.11.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Text Analysis - Topic Modelling
# ### <span style='color: green'>SETUP </span> Prepare and Setup Notebook <span style='float: right; color: red'>MANDATORY</span>

# %%

import os

import bokeh.plotting
import penelope.notebook.topic_modelling as gui
from IPython.display import display
from penelope.pipeline.config import CorpusConfig
from penelope.utility import pandas_utils

import __paths__  # pylint: disable=unused-import

bokeh.plotting.output_notebook()
pandas_utils.set_default_options()

current_state: gui.TopicModelContainer = gui.TopicModelContainer.singleton
corpus_folder: str = "/data/westac/sou_kb_labb"
corpus_config: CorpusConfig = CorpusConfig.load(os.path.join(__paths__.resources_folder, 'sou_sparv4.yml'))


# %% [markdown]
# ### <span style='color: green'>PREPARE</span> Load Topic Model <span style='float: right; color: red'>MANDATORY</span>

# %%
load_gui = gui.create_load_topic_model_gui(corpus_config, corpus_folder, current_state())
display(load_gui.layout())

# %% [markdown]
# ### <span style='color: green;'>BROWSE</span> Find topics by token<span style='color: red; float: right'>TRY IT</span>
#
# Displays topics in which given token is among toplist of dominant words.

# %%
gui.find_topic_documents_gui(
    current_state().inferred_topics.document_topic_weights, current_state().inferred_topics.topic_token_overview
)

# %% [markdown]
# ### <span style='color: green;'>BROWSE</span> Browse Topic Documents<span style='color: red; float: right'>TRY IT</span>
#
# Displays documents in which a topic occurs above a given threshold.

# %%
gui.display_topic_documents_gui(current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Display Topic's Word Distribution as a Wordcloud<span style='color: red; float: right'> TRY IT</span>

# %%
gui.display_topic_wordcloud_gui(current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic-Word Distribution<span style='color: red; float: right'>TRY IT</span>
#

# %%
gui.display_topic_word_distribution_gui(current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Trends over Time<span style='color: red; float: right'>RUN</span>

# %%
gui.display_topic_trends_gui(current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Trends Overview<span style='color: red; float: right'>TRY IT</span>
#
# - The topic shares  displayed as a scattered heatmap plot using gradient color based on topic's weight in document.
# - [Stanford’s Termite software](http://vis.stanford.edu/papers/termite) uses a similar visualization.

# %%
gui.display_topic_trends_overview_gui(current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Topic Network<span style='color: red; float: right'>TRY IT</span>
#
# Computes weighted graph of topics co-occurring in the same document. Topics are defined as co-occurring in a document if they both have a weight above given threshold. The edge weights are the number of co-occurrences (binary yes or no). Node size reflects topic proportions over the entire corpus computed in accordance to LDAvis topic proportions.

# %% code_folding=[0]
gui.display_topic_topic_network_gui(current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Document Topic Network<span style='color: red; float: right'>TRY IT</span>
#

# %%
gui.display_topic_document_network_gui(plot_mode=gui.PlotMode.Default, state=current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Focus-Topic Document Network<span style='color: red; float: right'>TRY IT</span>
#

# %%
gui.display_topic_document_network_gui(plot_mode=gui.PlotMode.FocusTopics, state=current_state())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic-Token  Network<span style='color: red; float: right'>TRY IT</span>

# %%

corpus_folder: str = "/data/westac/sou_kb_labb"
custom_styles = {'edges': {'curve-style': 'haystack'}}
w = gui.create_topics_token_network_gui(data_folder=corpus_folder, custom_styles=custom_styles)
display(w.layout())

# %%



import os
from dataclasses import dataclass
from typing import Dict

import pandas as pd
import penelope.common.goodness_of_fit as gof
import penelope.notebook.utility as notebook_utility
import penelope.notebook.word_trends as word_trends
from penelope import co_occurrence
from penelope.common.goodness_of_fit import GoodnessOfFitComputeError
from penelope.corpus import VectorizedCorpus
from penelope.notebook.ipyaggrid_utility import display_grid
from penelope.utility import getLogger

logger = getLogger()


@dataclass
class CoOccurrenceData:  # pylint: disable=too-many-instance-attributes
    corpus: VectorizedCorpus = None
    corpus_folder: str = None
    corpus_tag: str = None
    co_occurrences: pd.DataFrame = None
    compute_options: Dict = None
    co_occurrence_metadata: Dict = None
    goodness_of_fit: pd.DataFrame = None
    most_deviating_overview: pd.DataFrame = None
    most_deviating: pd.DataFrame = None


def compile_data(bundle: co_occurrence.Bundle, n_count: int = 5000) -> CoOccurrenceData:

    data = CoOccurrenceData(
        corpus=bundle.corpus,
        corpus_folder=bundle.corpus_folder,
        corpus_tag=bundle.corpus_tag,
        co_occurrences=bundle.co_occurrences,
        compute_options=bundle.compute_options,
    )

    if data.corpus is None:

        if data.co_occurrences is None:
            raise ValueError("Both corpus and co_occurrences cannot be None")

        data.corpus = co_occurrence.to_vectorized_corpus(
            co_occurrences=data.co_occurrences,
            document_index=data.corpus.document_index if data.corpus else None,
            value_column='value_n_t',
        ).group_by_year()

    data.goodness_of_fit = gof.compute_goddness_of_fits_to_uniform(data.corpus, None, verbose=False)
    data.most_deviating_overview = gof.compile_most_deviating_words(data.goodness_of_fit, n_count=n_count)
    data.most_deviating = gof.get_most_deviating_words(
        data.goodness_of_fit, 'l2_norm', n_count=n_count, ascending=False, abs_value=True
    )

    return data


def build_layout(data: CoOccurrenceData):  # pylint: disable=redefined-outer-name
    trends_with_pick_gui: word_trends.TrendsWithPickTokensGUI = word_trends.TrendsWithPickTokensGUI.create(
        data.corpus, tokens=data.most_deviating
    )

    tab_gof = (
        notebook_utility.OutputsTabExt(["GoF", "GoF (abs)", "Plots", "Slopes"], layout={'width': '98%'})
        .display_fx_result(0, display_grid, data.goodness_of_fit)
        .display_fx_result(1, display_grid, data.most_deviating_overview[['l2_norm_token', 'l2_norm', 'abs_l2_norm']])
        .display_fx_result(2, gof.plot_metrics, data.goodness_of_fit, plot=False, lazy=True)
        .display_fx_result(
            3, gof.plot_slopes, data.corpus, data.most_deviating, "l2_norm", 600, 600, plot=False, lazy=True
        )
    )

    layout = (
        notebook_utility.OutputsTabExt(["Data", "Explore", "Options", "GoF"], layout={'width': '98%'})
        .display_fx_result(0, display_grid, data.co_occurrences)
        .display_content(1, what=trends_with_pick_gui.layout(), clear=True)
        .display_as_yaml(2, data.compute_options, clear=True, width='800px', height='600px')
        .display_content(3, tab_gof, clear=True)
    )

    return layout


def loaded_callback(bundle: co_occurrence.Bundle, *_, **__):
    try:
        if os.environ.get('VSCODE_LOGS', None) is not None:
            logger.error("bug-check: vscode detected, aborting plot...")
            return

        data = compile_data(bundle=bundle)
        build_layout(data).display()
    except GoodnessOfFitComputeError as ex:
        logger.info(f"Unable to compute GoF: {str(ex)}")
    except Exception as ex:
        logger.exception(ex)



# # %%
# from  transformers import pipeline
# deps:
# transformers = "^4.3.2"
# torch = "^1.7.1"
# tensorflow = "^2.4.1"
# # https://github.com/search?l=Python&q=bert-base-swedish-cased-ner&type=Code

# def create_pipeline():
#     model = "KB/bert-base-swedish-cased-ner"
#     tokenizer = "KB/bert-base-swedish-cased-ner"
#     nlp = pipeline("ner", model=model, tokenizer=tokenizer)

#     entities = [nlp(sentence) for sentence in sentences]

# # %%

# import tensorflow as tf
# import os
# os.environ["CUDA_DEVICE_ORDER"]="PCI_BUS_ID"   # see issue #152
# #os.environ["CUDA_VISIBLE_DEVICES"]="-1"
# os.environ["CUDA_VISIBLE_DEVICES"] = "0,1,2,3,4,5"
# from tensorflow.python.client import device_lib
# print(device_lib.list_local_devices())

# # Allow memory growth for the GPU
# physical_devices = tf.config.experimental.list_physical_devices('GPU')
# print(physical_devices)
# #tf.config.experimental.set_memory_growth(physical_devices[0], True)
# #[![enter image description here][1]][1]

# # %%



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: 'Python 3.7.5 64-bit (''text_analytic_tools'': pipenv)'
#     language: python
#     name: python37564bittextanalytictoolspipenv8c3d4c9c6f39484cb74f0ad2d777602d
# ---

# %% [markdown]
# ## Text Analysis - Topic Modelling
# ### <span style='color: green'>SETUP </span> Prepare and Setup Notebook <span style='float: right; color: red'>MANDATORY</span>

# %%
# pylint: disable=wrong-import-position

import __paths__  # isort:skip pylint: disable=import-error, unused-import

import bokeh.plotting
import penelope.notebook.topic_modelling as gui
from IPython.core.interactiveshell import InteractiveShell
from IPython.display import display

import notebooks.political_in_newspapers.notebook_gui.publication_topic_network_gui as publication_topic_network_gui
import notebooks.political_in_newspapers.notebook_gui.topic_document_texts_gui as texts_gui
import notebooks.political_in_newspapers.notebook_gui.topic_topic_network_gui as topic_topic_gui
import notebooks.political_in_newspapers.notebook_gui.topic_trends_gui as trends_gui
import notebooks.political_in_newspapers.notebook_gui.topic_trends_overview_gui as overview_gui

InteractiveShell.ast_node_interactivity = "all"

# %matplotlib inline

current_state = gui.TopicModelContainer.singleton
bokeh.plotting.output_notebook()

# %% [markdown]
# ### <span style='color: green'>PREPARE</span> Load Topic Model <span style='float: right; color: red'>MANDATORY</span>

# %%

load_gui = gui.create_load_topic_model_gui(
    corpus_config=None, corpus_folder='/data/westac/political_in_newspapers', state=current_state()
)
display(load_gui.layout())

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Display Topic's Word Distribution as a Wordcloud<span style='color: red; float: right'> TRY IT</span>

# %%

try:
    gui.display_topic_wordcloud_gui(current_state())
except Exception as ex:
    print(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic-Word Distribution<span style='color: red; float: right'>TRY IT</span>
#

# %%

try:
    gui.display_topic_word_distribution_gui(current_state())
    # topic_word_distribution_gui.display_topic_tokens(current_state(), topic_id=0, n_words=100, output_format='Chart')
except Exception as ex:
    print(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Trends over Time<span style='color: red; float: right'>RUN</span>


# %%

try:
    trends_gui.display_gui(current_state())
    # trends_gui.display_topic_trend(current_state().inferred_topics.document_topic_weights, topic_id=0, year=None, year_aggregate='mean', output_format='Table')
except Exception as ex:
    print(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Trends Overview<span style='color: red; float: right'>TRY IT</span>
#
# - The topic shares  displayed as a scattered heatmap plot using gradient color based on topic's weight in document.
# - [Stanford’s Termite software](http://vis.stanford.edu/papers/termite) uses a similar visualization.

# %%

try:
    overview_gui.display_gui(current_state())
except ValueError as ex:
    print(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Publication Topic Network<span style='color: red; float: right'>TRY IT</span>
# The green nodes are documents, and blue nodes are topics. The edges (lines) indicates the strength of a topic in the connected document. The width of the edge is proportinal to the strength of the connection. Note that only edges with a strength above the certain threshold are displayed.

# %%

try:
    publication_topic_network_gui.display_gui(current_state())
except Exception as ex:
    print(ex)

# %% [markdown]
# ### <span style='color: green;'>BROWSE</span> Browse Topic Documents<span style='color: red; float: right'>TRY IT</span>
#
# Computes weighted graph of topics co-occurring in the same document. Topics are defined as co-occurring if they both exists  in the same document both having weights above threshold. Weight are number of co-occurrences (binary yes or no). Node size reflects topic proportions over the entire corpus (normalized document) length, and are computed in accordance to how node sizes are computed in LDAvis.

# %%

try:
    texts_gui.display_gui(current_state())
except Exception as ex:
    print(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic-Topic Network<span style='color: red; float: right'>TRY IT</span>
#
# Computes weighted graph of topics co-occurring in the same document. Topics are defined as co-occurring in a document if they both have a weight above given threshold. The edge weights are the number of co-occurrences (binary yes or no). Node size reflects topic proportions over the entire corpus computed in accordance to LDAvis topic proportions.


# %% code_folding=[0]

try:
    topic_topic_gui.display_gui(current_state())
except Exception as ex:
    print(ex)



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Most Discriminating Terms
#
# References:
#
#    King, Gary, Patrick Lam, and Margaret Roberts. "Computer-Assisted Keyword
#    and Document Set Discovery from Unstructured Text." (2014).
#    http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.458.1445&rep=rep1&type=pdf
#

# %%

from IPython.display import display
from penelope.notebook.mdw import main_gui

import __paths__

gui = main_gui.create_main_gui(corpus_folder=__paths__.data_folder)

display(gui)



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Concept Context Co-Occurrences Analysis

# %% [markdown]
# ### Setup notebook
# %%
# %load_ext autoreload
# %autoreload 2
# pylint: disable=too-many-instance-attributes, unused-argument

import importlib
import warnings
from typing import Optional

import penelope.notebook.co_occurrence.load_co_occurrences_gui as load_gui
import penelope.notebook.co_occurrence.to_co_occurrence_gui as compute_gui
from bokeh.plotting import output_notebook
from IPython.display import display
from penelope import co_occurrence, pipeline, workflows
from penelope.notebook.interface import ComputeOpts

import __paths__  # pylint: disable=unused-import

from .loaded_callback import loaded_callback

warnings.filterwarnings("ignore", category=FutureWarning)

output_notebook()

# %% [markdown]
# ### Generate new concept context co-co_occurrences
# %%


def compute_co_occurrence_callback(
    corpus_config: pipeline.CorpusConfig,
    args: ComputeOpts,
    checkpoint_file: Optional[str] = None,
) -> co_occurrence.ComputeResult:
    compute_result = workflows.co_occurrence.compute(
        args=args,
        corpus_config=corpus_config,
        checkpoint_file=checkpoint_file,
    )
    return compute_result


importlib.reload(compute_gui)
gui: compute_gui.ComputeGUI = compute_gui.create_compute_gui(
    corpus_config="riksdagens-protokoll",
    corpus_folder=__paths__.corpus_folder,
    data_folder=__paths__.data_folder,
    done_callback=loaded_callback,
    compute_callback=compute_co_occurrence_callback,
)
display(gui.layout())
# %% [markdown]
# ### Load saved concept context co-occurrences
# %%
lgu: load_gui.LoadGUI = load_gui.create_load_gui(
    data_folder=__paths__.data_folder, filename_pattern="*.co_occurrence.csv.zip", loaded_callback=loaded_callback
)
display(lgu.layout())



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Word, or part-of-word, trend analysis
#
# ### Text Processing Pipeline
#
# | 🔽 | Building block | Arguments | Description | Configuration
# | -- | :------------- | :------------- | :------------- | :------------- |
# | ⚙ | <b>SetTagger</b>SpacyModel | 'en' | Set PoS tagger | spaCy
# | 📜| <b>LoadText</b> | reader_opts, transform_opts | Text stream provider | config.yaml
# | 🔎 | <b>Tqdm</b> | ⚪ | Progress indicator | ⚪
# | ⌛ | <b>Passthrough</b> | ⚪ | Passthrough | ⚪
# | 🔨 | <b>ToTaggedFrame</b> | ⚪ Spacy | PoS tagging | config.yaml
# | 💾 | <b>Checkpoint</b> | checkpoint_filename | Checkpoint (tagged frames) to file | ⚪
# | 🔨 | TaggedFrame<b>ToTokens</b> | extract_tagged_tokens_opts, filter_opts | Tokens extractor | User specified
# | 🔨 | <b>TokensTransform</b> | tokens_transform_opts | Tokens transformer | User specified
# | 🔨 | <b>ToDocumentContentTuple</b> | ⚪ | API adapter | ⚪
# | 🔎 | <b>Tqdm</b> | ⚪ | Progress indicator | ⚪
# | 🔨 | <b>ToDTM</b> | vectorize_opts| DTM vectorizer | User specified
# | 💾 | <b>Checkpoint</b> | checkpoint_filename| Checkpoint (DTM) to file | User specified
#
# ### User instructions
#
# #### Compute DTM
#
# This notebook implements the entire processing pipeline from plain text to a computed (and stored)
# document-term matrix (DTM) that are the basis for the word trend exploration.
#
# For large corpora the DTM processing time can be considerable and in such a case you
# should consider using the CLI-version of the processing pipeline.
#
# Note that the computed DTM is saved on disk in the specified folder. You must enter a
# tag that will be used when naming the (principal) result data file. This file will be named
# "tag + `_vectorized_data.pickle`" and will be used to uniquely identify the bundle of files that makes
# up the result. Note that if the `tag` already exists in the specified target folder then it will
# be overwritten. You can use the tag to describe the arguments of the computation i.e. PoS tags etc. If the `Create folder` option is checked, then the result bundle will be stored in a subfolder of the target folder named _tag_.
#
# #### Compute arguments
#
# | | Config element |  Description |
# | -- | :------------- | :------------- |
# | | Corpus type | Type of corpus, disabled since only text corpora are allowed in this notebook.
# | | Source corpus file | Select file (ZIP) or folder that contains the text documents.
# | | Output tag | String that will be prefix to result files. Only valid filename chars allowed.
# | | Output folder | Target folder for result files.
# | | Part-of-speech groups | Groups of tags to include in DTM given corpus PoS-schema
# | | Remove stopwords | Remove common stopwords using NLTK language specific stopwords
# | | Extra stopwords | Additional stopwords
# | | Filename fields | Specifies attribute values to be extracted from filenames
#
# N.B. Note that PoS schema (e.g. SUC, Universal, ON5 Penn Treebank tag sets) and language must be set for each corpus.
#  This, and other options, is specified in the _corpus configuration file_. For an example, please see _SSI.yaml_ inf the `resources` folder.
#
# #### Load a DTM corpus
#
# To load an existing corpus youmust first select a file, then press <b>`Load`</b>. To select the file:</b> <b>1)</b> press <b>`Change`</b> to open the file browser, <b>2)</b> find and select the file you want to open and <b>3)</b> press <b>`Change`</b> again to confirm the file and close the file browser. Then you can load the corpus by pressing <b>`Load`</b>.
#
# #### Word trends
#
# Specifiy words of interest in the text box. You can use both wildcards and regular expressions to widen your search. The
# words in the vocabulary that matches what you have specified will be listed in the selection box. Since using wildcards and regexps can result
# in a large number of words, only the `Word count` matching most frequent words are displayed. Refine your search if you get to many matches.
# The words will be plotted when they are selected. You can select and plot multiple words by pressing CTRL when selected, or using arrow keys.
#
# The regular expressions must be surrounded by vertical bars `|`. To find words ending with `tion`
# you can enter `|.*tion$|` in the textbox. I might seem cryptical, but is a very powerful notation for searching words. The vertical
# bars is specified only so that the system can distinguish the regexp from "normal" words. The actual expression is `^.*tion$`.
# The dot and star`.*` matches any character (the dot) any number of times (the `*`). The dollar sign `$` indicates the word ending.
# So this expression matches all words that begins with any number of characters follwoed, by the character sequence `tion` at the end of the word.
# To match all words starting with `info`you can enter `|^info.*|` where `^` specifies the start of the word.
#
#
#
# %%


import penelope.notebook.word_trends.main_gui as main_gui
from bokeh.plotting import output_notebook
from IPython.core.display import display

import __paths__

output_notebook()
gui = main_gui.create_to_dtm_gui(
    corpus_config="riksdagens-protokoll",
    corpus_folder=__paths__.corpus_folder,
    data_folder=__paths__.data_folder,
    resources_folder=__paths__.resources_folder,
)
display(gui)

# %%
